package edu.msoe.se1021.Lab6;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.ConnectException;
import java.net.MalformedURLException;
import java.net.SocketTimeoutException;
import java.net.URL;
import java.net.URLConnection;
import java.net.UnknownHostException;

/**
 * This class will handle making connections with a website, downloading the conent, and will provide feedback on the performance of the website.
 * 
 * @author Walter Schilling
 *
 */
public class WebsiteTester {
	/**
	 * This variable holds the URL that is to be accessed.
	 */
	private URL myURL;
	/**
	 * This variable holds the connection to the URL.
	 */
	private URLConnection connection;
	/**
	 * This variable holds the content that is downloaded from the given URL.
	 */
	private String content;
	/**
	 * This si the timeout, or how long to wait, before indicating that the connection has failed.
	 */
	private int timeout = 10000;
	/**
	 * This variable indicates that the URL has been properly set.
	 */
	private boolean urlSet = false;
	/**
	 * This variable indicates how long it takes to download the given file from the website.
	 */
	private long downloadTime = 0;
	private int port;
	private String hostname;

	/**
	 * This method will convert a string into a URL that can be accessed.
	 * 
	 * @param url
	 *            This is a string representation for the URL. The string should
	 *            include
	 * @throws MalformedURLException
	 *             This exception will be thrown if the url passed in does not
	 *             represent a valid URL.
	 */
	public void openURL(String url) throws MalformedURLException {
		urlSet = false;
		myURL = new URL(url);
		urlSet = true;
	}

	/**
	 * This method will open a connection with a remote site in order that the
	 * page can be downloaded.
	 * 
	 * @throws MalformedURLException
	 *             This exception will be thrown if the url passed in does not
	 *             represent a valid URL.
	 * @throws UnknownHostException
	 *             This exception will be thrown if the system is unable to
	 *             connect to the remote host because while the URL is a vlaid
	 *             URL, it does not reference a machien which is online.
	 * @throws SocketTimeoutException
	 *             This exception will be thrown if it is taking too long to
	 *             connect to the given machine.
	 * @throws IOException
	 *             This exception will be thrown if there is another IO
	 *             exception opening the connection with the remote machine.
	 */
	public void openConnection() throws MalformedURLException,
			UnknownHostException, SocketTimeoutException, IOException {
		if (myURL == null) {
			throw new MalformedURLException("Invalid URL Entered.");
		}
		connection = myURL.openConnection();
		connection.setConnectTimeout(timeout);
		connection.connect();
		port = myURL.getPort();
		hostname = myURL.getHost();

	}

	/**
	 * 
	 * @throws UnknownHostException
	 *             This exception will be thrown if the system is unable to
	 *             connect to the remote host because while the URL is a vlaid
	 *             URL, it does not reference a machien which is online.
	 * @throws ConnectException
	 *             This exception will be thrown if the system is unable to
	 *             connect to the given URL and download it to the local
	 *             machine.
	 * @throws FileNotFoundException
	 *             This exception will be thrown if the given file does not
	 *             exist on the remote machien and therefore can not be
	 *             downloaded.
	 * @throws IOException
	 *             This exception will be thrown if there is a general problem
	 *             downloading the given file.
	 */
	public void downloadText() throws UnknownHostException, ConnectException,
			FileNotFoundException, IOException {
		long now = System.nanoTime();
		InputStream istream = connection.getInputStream();
		connection.setConnectTimeout(timeout);

		BufferedReader reader = new BufferedReader(new InputStreamReader(
				istream));
		StringBuilder out = new StringBuilder();
		String line;
		while ((line = reader.readLine()) != null) {
			out.append(line);
		}
		content = out.toString();
		reader.close();
		long time = System.nanoTime() - now;
		downloadTime = time / 1000000;
	}

	/**
	 * This method will return the size of the downloaded web resource.
	 * 
	 * @return The return will be the size of the web resource in bytes.
	 */
	public int getSize() {
		return content.length();
	}

	/**
	 * This method will set the timeout, or how long the system will wait for a
	 * connection from the remote server.
	 * 
	 * @param timeout
	 *            This is the timeout in ms.
	 * @throws NumberFormatException
	 *             A number format exception will be thrown if the entry is
	 *             either not a string or is a negative number.
	 */
	public void setTimeout(String timeout) throws NumberFormatException {
		int temp;
		try {
			temp = Integer.parseInt(timeout);
		} catch (NumberFormatException ex) {
			throw new NumberFormatException(
					"Invalid integer passed in as timeout");

		}

		if (temp < 0) {
			throw new NumberFormatException(
					"Timeout must be greater than or equal to 0.");
		}
		this.timeout = temp;
	}

	/**
	 * This method will return the current timeout as a string.
	 * 
	 * @return The timeout in ms will be returned as a string.
	 */
	public String getTimeout() {
		return "" + this.timeout;

	}

	/**
	 * This method will return the contents that ahve been downloaded from the
	 * given site.
	 * 
	 * @return The material downloaded will be returned as a string.
	 */
	public String getContent() {
		return content;
	}

	/**
	 * This method will obtain the download time in MS for the given file.
	 * 
	 * @return This is the time it takes to download the given file.
	 */
	public long getDownloadTime() {
		return downloadTime;
	}

	/**
	 * This method will return the Port that the system is connected to.
	 * 
	 * @return The port number will be returned.
	 */
	public int getPort() {
		return port;
	}

	/**
	 * This method will return the hostname for the given machine which is
	 * hosting the website.
	 * 
	 * @return The hostname will be returned as a string.
	 */
	public String getHostname() {
		return hostname;
	}

	/**
	 * This method will determine if the URL that is to be accessed has been
	 * initialized.
	 * 
	 * @return TRUE will be returned if a URL has been set. False otherwise.
	 */
	public boolean isUrlSet() {
		return urlSet;
	}

}
