package edu.msoe.se1021;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.DialogFragment;
import android.content.DialogInterface;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.EditText;
import com.example.Lab6.R;



/**
 * A two step dialog fragment that allows the user to modify a the timeout values if it is not sufficient.
 * @author Baohuy Ung
 */
public class SocketDialog extends DialogFragment{
    /**
     * Used for input for the adjusted time.
     */
    private EditText time;
    /**
     * Holds the second dialog box while the first is displayed
     */
    private AlertDialog.Builder timeExtention;
    /**
     * Holds an instance of the call back interface for the click listener
     */
    private TimeUpdate tu;

    /**
     * Initializes the first dialog
     * @param saveInstanceState store any previous states
     * @return returns the completed dialog object
     */
    @Override
    public Dialog onCreateDialog(Bundle saveInstanceState){
        //creates the second dialog box and stores it into timeExtention. It is shown when the user clicks the "yes" option
        timeExtensionDialog();
        //sets dialog properties
        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
        builder.setTitle("Connection Timeout")
                .setMessage("There has been a timeout reaching the site. " +
                        "Would you like to extend the timeout period? ");
        builder.setPositiveButton("Yes", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                dismiss(); //close first dialog
                timeExtention.show(); //open second dialog
            }
        });
        builder.setNegativeButton("No", null);
        return builder.create();
    }

    /**
     * Creates the second dialog that allows the user to adjust the timeout value
     */
    public void timeExtensionDialog(){
        timeExtention = new AlertDialog.Builder(getActivity());
        LayoutInflater li = getActivity().getLayoutInflater();
        View view = li.inflate(R.layout.time_extension,null);
        timeExtention.setView(view);
        time = (EditText)view.findViewById(R.id.Time);
        timeExtention.setTitle("Input");
        timeExtention.setNegativeButton("Cancel", null);
        timeExtention.setPositiveButton("OK", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                //calls the call back method and passes the values back to the activity
                tu.updateTimeout(time.getText().toString());
            }
        });
        timeExtention.create();//creates the second dialog, however it is hidden until the first dialog is dismissed
    }

    /**
     * allows use to set up a call back
     * @param tu
     */
    public void setTime(TimeUpdate tu) {
        this.tu = tu;
    }

    /**
     * Call back interface to retrieve values set in the dialog
     */
    public interface TimeUpdate{
        public void updateTimeout(String time);
    }

}
