package edu.msoe.se1021;

import android.app.Activity;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import com.example.Lab6.R;
import edu.msoe.se1021.Lab6.WebsiteTester;

//Exception imports
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;

/**
 * Lab 6 app. Downloads a webpage and displays feedback on the process.
 * @author Baohuy Ung
 */
public class WebsiteTesterActivity extends Activity{

    private EditText url;
    private Button analyze;
    private EditText scrollPanel;
    private EditText resourceSize;
    private EditText downloadTime;
    private EditText host;
    private EditText port;
    private EditText timeout;
    private Button setTimeoutButton;
    /**
     * WebsiteTester object will handle the retrieve of a webpage and throw appropriate exceptions.
     */
    private WebsiteTester webTester;
    /**
     * A generic dialog box which the user can use the display relevant information.
     */
    private ErrorDialog dialog;

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        init();
    }

    /**
     * Initializes the GUI components
     */
    private void init(){
        webTester = new WebsiteTester();
        dialog = new ErrorDialog();
        findIds();
        // TODO Set timeout field to contain the Timeout value stored in the
        // TODO webTester object.

        setTimeClick();
        setAnalyzeClick();

    }

    /**
     * Links all the GUI components to their components in the layout.
     */
    private void findIds(){
        url = (EditText)findViewById(R.id.URLTextBox);
        analyze = (Button)findViewById(R.id.AnalyzeButton);
        scrollPanel = (EditText)findViewById(R.id.ScrollPanel);
        resourceSize = (EditText)findViewById(R.id.ResourceSize);
        downloadTime = (EditText)findViewById(R.id.DownloadTime);
        host = (EditText)findViewById(R.id.HostName);
        port = (EditText)findViewById(R.id.PortNumber);
        timeout = (EditText)findViewById(R.id.TimeoutValue);
        setTimeoutButton = (Button)findViewById(R.id.Set);
    }

    /**
     * Sets up the timeout field and adds the click listener to it.
     */
    private void setTimeClick() {
        setTimeoutButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                // TODO If an invalid timeout is sent to WebsiteTester.setTimeout(), an exception is thrown.
                // TODO Handle the exception in this method.
                // TODO  1. Use android.util.Log.e() method to log the exception
                // TODO  2. Display an Android toast with the appropriate error message using the
                // TODO     the dialog object (from the edu.msoe.se1021.ErrorDialog class.
                // TODO     To do this, first call the setProperties() method and then call:
                // TODO       dialog.show(getFragmentManager(), "Dialog");
                // TODO  3. Reset the timeout field to contain the timeout value currently stored
                // TODO     in the WebsiteTester object.

                webTester.setTimeout(timeout.getText().toString());
                Toast toast = Toast.makeText(getApplicationContext(), "Timeout set: " + timeout.getText().toString() + "ms", Toast.LENGTH_SHORT);
                toast.show();
            }
        });
    }

    /**
     * Setting up the analyze button by adding the click listener to the analyze button
     * that starts the webpage retrieval.
     */
    private void setAnalyzeClick(){
        analyze.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View v) {
                URLGrab task = new URLGrab();
                task.execute();
            }
        });
    }

    /**
     * Is called only after the website is successfully retrieved to update the text fields
     */
    private void updateDisplay() {
        resourceSize.setText("" + webTester.getSize());
        scrollPanel.setText(scrollPanel.getText() + "\n" + webTester.getContent());
        downloadTime.setText("" + webTester.getDownloadTime() + " ms");
        host.setText(webTester.getHostname());
        port.setText(""+ webTester.getPort());
    }

    /**
     * Used to create specific dialogs for different exceptions.
     */
    private static final int NO_EXCEPTION = 0;
    private static final int MALFORMED_URL = 1;
    private static final int SOCKET_TIMEOUT = 2;
    private static final int FILE_NOT_FOUND = 3;
    private static final int UNKNOWN_HOST = 4;
    private static final int IO_EXCEPTION = 5;

    /**
     * Class to start an asynchronous task of grabbing information from the webpage.
     * Needs to be async because services (background tasks like internet related items)
     * should not be performed on the main GUI.
     * Implements the SocketDialog.TimeExtentionUpdate interface as a call back to get
     * information from the dialogs.
     */
    private class URLGrab extends AsyncTask<Void, Void, Integer>{

        /**
         * Threaded event in which we obtain the information about the webtsite
         * @param params
         * @return specific exception value
         */
        @Override
        protected Integer doInBackground(Void... params) {
            // TODO In Android, we need to run the analyze request as a separate
            // TODO task.  This method should make the appropriate requests to
            // TODO the WebsiteTester object.  In particular, you will need to
            // TODO open the URL, open the connection, and then download the text.
            // TODO
            // TODO The WebsiteTester class may throw one of a number of different
            // TODO types of exceptions.  In this method, you will need to catch
            // TODO and log any exception that is thrown and the return the appropriate
            // TODO class constant (e.g., MALFORMED_URL, SOCKET_TIMEOUT, FILE_NOT_FOUND, etc...)
            // TODO so that the openDialog() method (see below) can display the appropriate
            // TODO dialog indicating what error was encountered.
            // TODO
            // TODO If no exceptions are thrown, then the method should return the
            // TODO NO_EXCEPTION class constant.

            return NO_EXCEPTION;
        }

        /**
         * When the website is retrieved if, there is either an exception or no exception.
         * This method is run only after the threaded (website grab) event is finished.
         * @param type of exception thrown if any
         */
        @Override
        protected void onPostExecute(Integer type) {
            dialogOpen(type);
        }

    }

    /**
     * Called after the async task (website grab) is complete, it will update the appropriate dialog
     * for the exception thrown.
     * @param type which type of exception was thrown
     */
    private void dialogOpen(int type) {
        // TODO Based on the type, the appropriate dialog should be displayed.  Also, any
        // TODO UI elements that need to be updated in response to an error, should be updated
        // TODO in this method.  (See the assignment page for specifics on what needs to be
        // TODO updated.  If the type is NO_EXCEPTION, then the updateDisplay() method must be
        // TODO called.
        // TODO
        // TODO The dialog object from the ErrorDialog class must be used to display the
        // TODO appropriate error message to the user in an Android toast.
        // TODO
        // TODO If a socket timeout exception was thrown, you'll need to deal with it slightly
        // TODO differently because handling this type of exception requires that you ask
        // TODO additional information from the user.  In particular, you should give the user
        // TODO the option to extend the timeout period.  The SocketDialog class is provided
        // TODO to do this work.  You can make use of it with the following code:
        // TODO
        // TODO        SocketDialog socketDialog = new SocketDialog();
        // TODO        //creates a call back that passes the values from the dialog back to the main activity
        // TODO        socketDialog.setTime(new SocketDialog.TimeUpdate() {
        // TODO            @Override
        // TODO            public void updateTimeout(String time) {
        // TODO                webTester.setTimeout(time);
        // TODO                timeout.setText(time);
        // TODO            }
        // TODO        });
        // TODO        socketDialog.show(getFragmentManager(), "Socket Dialog");

    }
}


